/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"
#include "filterkit.h"
#include "messagebus.h"
#include "util.h"

int ParseFilterConf(string filter_str, filter_set *in_set, 
					GlobalRegistry *globalreg) {
    size_t parse_pos = 0;
    size_t parse_error = 0;
	char errstr[1024];

    while (parse_pos < filter_str.length()) {
        size_t addr_term_end;
        size_t address_target = 0; // 1=bssid 2=source 4=dest 7=any

        if (filter_str[parse_pos] == ',' || filter_str[parse_pos] == ' ') {
            parse_pos++;
            continue;
        }

        if ((addr_term_end = filter_str.find('(', parse_pos + 1)) == string::npos) {
			snprintf(errstr, 1024, "Could not parse filter line '%s', no '(' found",
					 filter_str.c_str());
			globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
            parse_error = 1;
            break;
        }

        string addr_term = StrLower(filter_str.substr(parse_pos, 
													  addr_term_end - parse_pos));

        parse_pos = addr_term_end + 1;

        if (addr_term.length() == 0) {
			snprintf(errstr, 1024, "Could not parse filter line '%s', no address "
					 "given.", filter_str.c_str());
			globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
            parse_error = 1;
            break;
        }

        if (addr_term == "any") {
            address_target = 7;
        } else if (addr_term == "bssid") {
            address_target = 1;
        } else if (addr_term == "source") {
            address_target = 2;
        } else if (addr_term == "dest") {
            address_target = 4;
        } else {
			snprintf(errstr, 1024, "Could not parse filter line, unknown address "
					 "type '%s'", addr_term.c_str());
			globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
            parse_error = 1;
            break;
        }

        if ((addr_term_end = filter_str.find(')', parse_pos + 1)) == string::npos) {
			snprintf(errstr, 1024, "Could not parse filter line '%s', no ')' found",
					 filter_str.c_str());
			globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
            parse_error = 1;
            break;
        }

        string term_contents = filter_str.substr(parse_pos, 
												 addr_term_end - parse_pos);

        parse_pos = addr_term_end + 1;

        if (term_contents.length() == 0) {
			snprintf(errstr, 1024, "Could not parse filter line '%s', no addresses "
					 "listed.", filter_str.c_str());
			globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
            parse_error = 1;
            break;
        }

        size_t term_parse_pos = 0;
        while (term_parse_pos < term_contents.length()) {
            size_t term_end;
            size_t invert = 0;

            if (term_contents[term_parse_pos] == ' ' || 
				term_contents[term_parse_pos] == ',') {
                term_parse_pos++;
                continue;
            }

            if (term_contents[term_parse_pos] == '!') {
                invert = 1;
                term_parse_pos++;
            }

            if ((term_end = term_contents.find(',', term_parse_pos + 1)) == 
				string::npos)
                term_end = term_contents.length();

            string single_addr = term_contents.substr(term_parse_pos, 
													  term_end - term_parse_pos);

            mac_addr mac = single_addr.c_str();
            if (mac.error != 0) {
				snprintf(errstr, 1024, "Could not parse filter MAC address '%s'",
						 single_addr.c_str());
				globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
                parse_error = 1;
                break;
            }

            // Catch non-inverted 'ANY'
            if (address_target == 7 && invert == 0) {
				snprintf(errstr, 1024, "Filtering type 'ANY' will discard all "
						 "packets.  The 'ANY' filter type can only be used with "
						 "inverted matches.");
				globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
                parse_error = 1;
                break;
            }

            // Insert it into the map, we'll look later to see if it's an 
			// inversion collision
            if (address_target & 0x01) {
                in_set->bssid_filter.insert(mac, invert);
            } if (address_target & 0x02) {
                in_set->source_filter.insert(mac, invert);
            } if (address_target & 0x04) {
                in_set->dest_filter.insert(mac, invert);
            }

            term_parse_pos = term_end + 1;
        }

    }

    if (parse_error == 1)
        return -1;

    int inversion_tracker = -1;

    for (macmap<int>::iterator x = in_set->bssid_filter.begin(); 
		 x != in_set->bssid_filter.end(); x++) {

        if (inversion_tracker == -1) {
            inversion_tracker = *x->second;
            continue;
        }

        if (*x->second != inversion_tracker) {
			snprintf(errstr, 1024, "BSSID filter has an illegal mix of normal and "
					 "inverted addresses.  All addresses in a filter term must be "
					 "inverted or all must be normal.");
			globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
            return -1;
        }
    }
    in_set->bssid_invert = inversion_tracker;

    inversion_tracker = -1;
    for (macmap<int>::iterator x = in_set->source_filter.begin(); 
		 x != in_set->source_filter.end(); x++) {

        if (inversion_tracker == -1) {
            inversion_tracker = *x->second;
            continue;
        }

        if (*x->second != inversion_tracker) {
			snprintf(errstr, 1024, "SOURCE filter has an illegal mix of normal and "
					 "inverted addresses.  All addresses in a filter term must be "
					 "inverted or all must be normal.");
			globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
            return -1;
        }
    }
    in_set->source_invert = inversion_tracker;

    inversion_tracker = -1;
    for (macmap<int>::iterator x = in_set->dest_filter.begin(); 
		 x != in_set->dest_filter.end(); x++) {

        if (inversion_tracker == -1) {
            inversion_tracker = *x->second;
            continue;
        }

        if (*x->second != inversion_tracker) {
			snprintf(errstr, 1024, "DEST filter has an illegal mix of normal and "
					 "inverted addresses.  All addresses in a filter term must be "
					 "inverted or all must be normal.");
			globalreg->messagebus->InjectMessage(errstr, MSGFLAG_ERROR);
            return -1;
        }
    }
    in_set->dest_invert = inversion_tracker;

	return 1;
}


